@@ -316,8 +316,9 @@ def add_source_roots(paths: List[Union[Path, str]]) -> None:
         sys.path.insert(0, str(p))
 
 
-def get_src_root(name: str) -> Path:
-    path = Path(".").absolute()
+def get_src_root(name: str, file: Optional[str] = None) -> Path:
+    root = Path(file).parent if file else "."
+    path = Path(root).absolute()
 
     while not list(path.glob(f"*{name}")):
         if path == path.parent:
