@@ -19,7 +19,7 @@ from xonsh.ptk_shell.shell import PromptToolkitShell
 from xonsh.readline_shell import ReadlineShell
 
 import envo
-from envo import logger
+from envo import comp, logger
 from envo.misc import Callback, is_windows
 
 
@@ -28,6 +28,12 @@ class PromptState(Enum):
     NORMAL = 1
 
 
+@dataclass
+class ShellCodeError(Exception):
+    msg: str
+    exc: Exception
+
+
 class PromptBase:
     default: str = str(DEFAULT_PROMPT)
     loading: bool = False
@@ -177,7 +183,20 @@ class Shell(BaseShell):  # type: ignore
 
         from xonsh.codecache import run_compiled_code
 
-        run_compiled_code(c, self.ctx, None, "single")
+        exc = None
+        value = None
+        traceback = None
+
+        if comp.old_xonsh:
+            try:
+                run_compiled_code(c, self.ctx, None, "single")
+            except Exception:
+                exc, value, traceback = sys.exc_info()
+        else:
+            exc, value, traceback = run_compiled_code(c, self.ctx, None, "single")
+
+        if exc:
+            raise ShellCodeError(msg=f'"{code}" -> {repr(value)}', exc=exc)
 
     def run_code(self, code: str) -> None:
         logger.debug(f'Running code """{code}"""')
@@ -203,7 +222,7 @@ class Shell(BaseShell):  # type: ignore
         import signal
 
         import xonsh.history.main as xhm
-        from xonsh.built_ins import XonshSession, load_builtins
+        from xonsh.built_ins import XSH
         from xonsh.imphooks import install_import_hooks
         from xonsh.xontribs import xontribs_load
 
@@ -212,13 +231,11 @@ class Shell(BaseShell):  # type: ignore
         data_dir = Path.home() / f".envo/xonsh_data/{data_dir_name}"
         os.makedirs(data_dir, exist_ok=True)
 
-        execer = Execer(xonsh_ctx=ctx)
+        execer = Execer()
 
-        builtins.__xonsh__ = XonshSession(ctx=ctx, execer=execer)  # type: ignore
+        XSH.load(execer=execer, ctx=ctx)
 
-        load_builtins(ctx=ctx, execer=execer)
-        env = builtins.__xonsh__.env  # type: ignore
-        env.update(
+        XSH.env.update(
             {
                 "XONSH_INTERACTIVE": True,
                 "SHELL_TYPE": "prompt_toolkit",
@@ -227,13 +244,16 @@ class Shell(BaseShell):  # type: ignore
             }
         )
 
-        if "ENVO_SHELL_NOHISTORY" not in os.environ:
-            builtins.__xonsh__.history = xhm.construct_history(  # type: ignore
-                env=env.detype(), ts=[time.time(), None], locked=True, buffersize=100
-            )
-            builtins.__xonsh__.history.gc.wait_for_shell = False  # type: ignore
+        XSH.history = xhm.construct_history(  # type: ignore
+            env=XSH.env.detype(), ts=[time.time(), None], locked=True, buffersize=100
+        )
+        XSH.history.gc.wait_for_shell = False  # type: ignore
+
+        if comp.old_xonsh:
+            install_import_hooks()
+        else:
+            install_import_hooks(execer)
 
-        install_import_hooks()
         builtins.aliases.update({"ll": "ls -alF"})  # type: ignore
         xontribs_load([""])
 
@@ -245,8 +265,8 @@ class Shell(BaseShell):  # type: ignore
             signal.signal(signal.SIGTTOU, func_sig_ttin_ttou)
 
         shell = cls(calls, execer)
-        builtins.__xonsh__.shell = shell  # type: ignore
-        builtins.__xonsh__.shell.shell = shell  # type: ignore
+        XSH.shell = shell  # type: ignore
+        XSH.shell.shell = shell  # type: ignore
 
         return shell
 
@@ -269,7 +289,7 @@ class Shell(BaseShell):  # type: ignore
         try:
             # W want to catch all exceptions just in case the command fails so we can handle std_err and post_cmd
             self.append_history = BaseShell._append_history
-            BaseShell._append_history = lambda self, inp, ts, spc, tee_out: None
+            BaseShell._append_history = lambda self, inp, ts, spc, tee_out, cwd: None
             ts0 = time.time()
 
             if line:
@@ -353,7 +373,6 @@ class FancyShell(Shell, PromptToolkitShell):  # type: ignore
         self.calls.on_ready()
 
         self.cmdloop()
-
         self.calls.on_exit()
 
     def set_prompt(self, prompt: str) -> None:
@@ -361,10 +380,6 @@ class FancyShell(Shell, PromptToolkitShell):  # type: ignore
         self.prompter.message = self.prompt_tokens()
         self.prompter.app.invalidate()
 
-    def redraw(self) -> None:
-        self.prompter.app.renderer.erase(leave_alternate_screen=False)
-        self.prompter.app.invalidate()
-
 
 class SimpleShell(Shell, ReadlineShell):  # type: ignore
     def start(self) -> None:
